
\name{mcmc_sl}
\alias{mcmc_sl}
\title{Markov chain Monte Carlo sampler with the surrogate likelihood}
\description{
Draw a sample for each vertex from its posterior distribution based on the surrogate likelihood, using Metropolis-Hastings alrigothm.
}

\usage{
mcmc_sl(A, d, sigma = 1, nchain = 10000, burnin = 2000, thinning = 1, parallel = TRUE)
}

\arguments{
  \item{A}{an adjacency matrix. For now, only binary and symmetric matrices are allowed.}
  \item{d}{the embedding dimension.}
  \item{sigma}{tuning parameter for the covariance of the normal random walk proposal in the Metropolis-Hastings algorithm.}
  \item{nchain}{the length of the output chain after thinning.}
  \item{burnin}{the length of the burnin portion after thinning.}
  \item{thinning}{the thinning width. The number of sample points actually drawn in the algorithm is thinning * (nchain + burnin).}
  \item{parallel}{if TRUE, the algorithm is run parallel over the vertices.}
}

\details{The surrogate likelihood of random dot product graphs and the associated methods for estimating the latent positions are proposed by Wu and Xie (2022).
}

\value{
\item{post_mean}{the mean of \code{chain_array} (i.e., the sample from the posterior distribution), arranged as a matrix of dimension n by d, where n is the number of vertices and d is the embedding dimension.}
\item{accept_rate}{a vector which records the accept rate in the Metropolis-Hastings algorithm of each vertex.}
\item{chain_array}{the output Markov chain, arranged as an array of dimension c(d, nchain, n).}
}

\references{
\doi{https://doi.org/10.1016/S0304-4076(03)00100-3}
\doi{https://doi.org/10.48550/arXiv.2203.16688}
}

\note{}

\author{}


\seealso{
\code{\link{abind}}
\code{\link{ase}}
\code{\link{ose}}
\code{\link{mclapply}}
}

\examples{
## set the number of vertices
n = 100

## create an adjacency matrix
m = matrix(data = rbinom(n = n*n, size = 1, prob = 0.5), nrow = n)
m[lower.tri(m)] = t(m)[lower.tri(m)]
}

\keyword{ase}
\keyword{ose}


