
\name{msle_sgd}
\alias{msle}
\title{Maximum surrogate likelihood estimate using stochastic gradient descent}
\description{
Compute the maximum surrogate likelihood estimate from an adjacency matrix of a random dot product graph, using stochastic gradient descent.
}

\usage{
msle_sgd(A, d,
        a_0 = 1, b_0 = 1, epsilon = 0.5, tolerance = 1,
        batch_size = 1, max_nepoch = 1e5, show_message = FALSE)
}

\arguments{
  \item{A}{an adjacency matrix. For now, only binary and symmetric matrices are allowed.}
  \item{d}{the embedding dimension.}
  \item{a_0}{tuning parameter for stochastic gradient descent.}
  \item{b_0}{tuning parameter for stochastic gradient descent.}
  \item{epsilon}{tuning parameter for stochastic gradient descent.}
  \item{tolerance}{the criterior of convergence. When the l2 norm of the current gradient is less than this number, convergence is attained and the current iterate is used as the estimate.}
  \item{batch_size}{the size of the batch in stochastic gradient descent. Should be between 1 and the sample size.}
  \item{max_nepoch}{the maximum number of epochs that should be run.}
  \item{show_message}{if TRUE, a message shows up each time the estimate of a vertex is complete.}
}

\details{The surrogate likelihood of random dot product graphs and the associated methods for estimating the latent positions are proposed by Wu and Xie (2022).

The stochastic gradient descent algorithm with adaptive step sizes in the implementation is proposed by Li and Orabona (2019).
}

\value{
\code{msle_sgd} returns the latent positions of the vertices in the Euclidean space, arranged in a matrix of dimension n by d, where n is the number of vertices and d is the embedding dimension.
}

\references{
\url{http://jmlr.org/papers/v12/duchi11a.html}
\doi{https://doi.org/10.48550/arXiv.1805.08114}
}

\note{}

\author{}


\seealso{
\code{\link{ase}}
\code{\link{ose}}
}

\examples{
## set the number of vertices
n = 100

## create an adjacency matrix
m = matrix(data = rbinom(n = n*n, size = 1, prob = 0.5), nrow = n)
m[lower.tri(m)] = t(m)[lower.tri(m)]

## the maximum surrogate likelihood estimate of the latent position matrix, with embedding dimension being 2
msle_sgd(A = m, d = 2, a_0 = 0.1, b_0 = 10, epsilon = 0.1, tolerance = 1e-3,
         batch_size = 10, max_nepoch = 1e5, show_message = TRUE)
}

\keyword{ase}
\keyword{ose}


