
\name{ose}
\alias{ose}
\title{One-Step Estimate}
\description{
Compute the one-step estimate from an adjacency matrix of a random dot product graph, based on the adjacency spectral embedding.
}

\usage{
ose(A, d)
}

\arguments{
  \item{A}{an adjacency matrix. For now, only binary and symmetric matrices are allowed.}
  \item{d}{the embedding dimension.}
}

\details{The one-step procedure for estimating the latent positions of random dot product graphs is proposed by Xie and Xu (2021).}

\value{
\code{ose} returns the latent positions of the vertices in the Euclidean space, arranged in a matrix of dimension n by d, where n is the number of vertices and d is the embedding dimension.
}

\references{
\doi{https://doi.org/10.1080/01621459.2021.1948419}
}

\note{}

\author{}


\seealso{\code{\link{ase}}}

\examples{
## set the number of vertices
n = 100

## create an adjacency matrix
m = matrix(data = rbinom(n = n*n, size = 1, prob = 0.5), nrow = n)
m[lower.tri(m)] = t(m)[lower.tri(m)]

## the one-step estimate of the latent position matrix, with embedding dimension being 2
ose(A = m, d = 2)
}

\keyword{ase}


